Import ""
  GetNativeSystemInfo(*info)
EndImport

IncludeFile "ClassCompanyTables.pb"
IncludeFile "LoadDriver.pb"

Structure PciDevData
Pcibus.s
PciDevice.s
PciFunction.s
VendorID.s
VendorDescr.s
DeviceID.s
StructureUnion
	DeviceDescr.s
	*pDeviceDescr
EndStructureUnion
Revision.s
SubVendorID.s
SubVendorDescr.s
SubDeviceID.s
Class.s
StructureUnion
	ClassDescr.s
	*pClassDescr
EndStructureUnion
IRQ.s
CacheLine.s
Latency.s
Header.s
EndStructure

Global Dim DeviceData.PciDevData(1)
Global NumberOfBus.l  =  256
Global NumberOfDevice.l=  32
Global NumberOfFunc.l  =   8

Procedure.l FindPCIClass(baseclass.b, subclass.b, programif.b, index.l)
	count.l=0
	pci_address.l
	Dim conf.l(2)
	ret.l
	multifuncflag.l
	type.b
	bus.l: dev.l: func.l

	For bus = 0 To NumberOfBus - 1
		For dev = 0 To NumberOfDevice - 1
			multifuncflag = 0
			For func = 0 To NumberOfFunc - 1
				If multifuncflag = 0 And func > 0: Break: EndIf

				pci_address = ((bus & $ff)<<8) | ((dev & $1f)<<3) | (func & 7)
				ret = pciConfigRead(pci_address, 0, @conf(), SizeOf(Long) * 3)
				If ret = 0
					If func = 0
						If pciConfigRead(pci_address, $0e, @type, 1) = 0
							If type & $80: multifuncflag = 1:EndIf
						EndIf
					EndIf
					If conf(2) & $ffffff00 = (baseclass<<24) | (subclass<<16) | (programif<<8)
						If count = index: ProcedureReturn pci_address<<16: EndIf
						count+1
						Continue
					EndIf
				ElseIf ret = PCI_ERR_BUSNOTEXIST
					ProcedureReturn $86
				ElseIf ret = PCI_ERR_NODEVICE And func = 0
					Break
				EndIf 
			Next
		Next
	Next
	ProcedureReturn $86
EndProcedure

Procedure.l DeviceDescription(DeviceCode.s, *ChipName.STRING, *ChipDescr.STRING)
	Protected file.l = ReadFile(#PB_Any, GetPathPart(ProgramFilename()) + "pcinames.txt")
	;DevName.s
	Protected chip.s
	DeviceCode = UCase(DeviceCode)
	If Not file
		*ChipName\s = "Not available";Error opening file
		*ChipDescr\s = *ChipName\s
		ProcedureReturn 1
	EndIf
	;Protected stringFormat.l = ReadStringFormat(file)		;ASCII, UTF8, UTF16
	;Shared GetTextData()
	While Not Eof(file)
		Protected line.s = ReadString(file, stringFormat)
		If UCase(Left(line,8)) = DeviceCode
			chip = line
			Break
		EndIf
	Wend
	CloseFile(file)
	;i=FindString(chip,Chr(9),1)
	If chip<>""
		chip = Mid(chip,10)
		i=FindString(chip,Chr(9),1)
		If i=1;no chip name
			*ChipName\s = Mid(chip,2)
			*ChipDescr\s = *ChipName\s
		Else
			*ChipName\s = Left(chip,i-1)
			*ChipDescr\s = Mid(chip,i+1)
		EndIf
	Else
		*ChipName\s = "Not available";chip not found
		*ChipDescr\s = *ChipName\s
		ProcedureReturn 2
	EndIf
	ProcedureReturn 0
EndProcedure

Procedure ChipException(NorthID.s, SouthID.s, NorthRevID.s, *North.STRING, *South.STRING)
*North\s="":*South\s="";initialization
; E7210 Or 875P
If (NorthID = "80862578")
	If (SouthID = "808625A1") ;  Intel 6300ESB
		*North\s ="Intel E7210"
	Else
		*North\s ="Intel 875P"
	EndIf
EndIf
; For VIA C7
If NorthID = "11060314" And Family = 6 : *North\s ="CN700": EndIf
; 845D or 845MP
 If (NorthID = "80861A30" And 4 <= Val(NorthRevID) And Val(NorthRevID) < 11) :
     If (SouthID = "8086248C") ;82801CAM (ICH3-M)
         *North\s ="82845MP"
     Else
         *North\s ="82845D"
     EndIf
 EndIf
;     852GM or 855GM
If (NorthID = "80863580")
	If (Family = $F); Pentium 4-M
		*North\s ="Intel 852GM (Montara-GML)"
	Else ; Pentium M
		*North\s ="Intel 855GM (Montara-GM)"
	EndIf
EndIf
If Left(NorthID, 7) = "808684C" : *North\s ="450KX/GX/NX (Mars/Orion/?)": EndIf
If Left(NorthID, 7) = "808684E" : *North\s ="460GX": EndIf
 ;    South
If (SouthID = "10395513" Or SouthID = "10390008")
	If (NorthID = "10390658") : *South\s = "SiS 963": EndIf
	If (NorthID = "10390655") : *South\s = "SiS 963": EndIf
	If (NorthID = "10390651") : *South\s = "SiS 961/962": EndIf
	If (NorthID = "10390650") : *South\s = "SiS 961/962": EndIf
	If (NorthID = "10390648") : *South\s = "SiS 963": EndIf
	If (NorthID = "10390646") : *South\s = "SiS 961/962": EndIf
	If (NorthID = "10390645") : *South\s = "SiS 961": EndIf
	If (NorthID = "10390755") : *South\s = "SiS 963": EndIf
	If (NorthID = "10390746") : *South\s = "SiS 963": EndIf
	If (NorthID = "10390740") : *South\s = "SiS 961": EndIf
EndIf
; SiS 745
If (NorthID = "10390745") : *South\s = "SiS 745": EndIf
; SiS 735
If (NorthID = "10390735") : *South\s = "SiS 735": EndIf
; SiS 733
If (NorthID = "10390733") : *South\s = "SiS 733": EndIf
; SiS 730
If (NorthID = "10390730") : *South\s = "SiS 730": EndIf
; SiS 630
If (NorthID = "10390630") : *South\s = "SiS 630": EndIf
; SiS 633
If (NorthID = "10390633") : *South\s = "SiS 633": EndIf
; SiS 635
If (NorthID = "10390635") : *South\s = "SiS 635": EndIf
; ALi
If NorthID = "10B91531" And SouthID = "10B91533" : *North\s ="Aladdin IV": EndIf
If NorthID = "10B91531" And SouthID = "10B91543" : *North\s ="Aladdin IV+": EndIf
; VIA
If (NorthID = "11063189" And NorthRevID = "00") : *North\s ="VT8377  ( KT400 )": EndIf
If (NorthID = "11063189" And NorthRevID = "80") : *North\s ="VT8377  ( KT400A )": EndIf
EndProcedure

Procedure.l SearchPCIDevices();(Array DeviceData.PciDevData(1))
	OnErrorGoto(?ExitDevs)
	bus.l: dev.l: func.l
	pci_address.l
	id.l
	max_bus.l
	DetectedDevice.l = 0
	type.b
	vender_id.l
	max_bus = NumberOfBus
	IDD.l
	North.STRING
	South.STRING
	ChipName.STRING
	ChipDescr.STRING
	Protected tmp.s
	For bus = 0 To max_bus-1
		For dev = 0 To NumberOfDevice-1
			For func = 0 To NumberOfFunc-1
				pci_address = ((bus & $ff)<<8) | ((dev & $1f)<<3) | (func & 7)
				id = PciConfigReadLong(pci_address, 0)
				vender_id = id & $ffff;
				If vender_id <> $ffff And vender_id <> 0
					Devicedata(DetectedDevice)\PciBus = Str(bus)
					Devicedata(DetectedDevice)\PciDevice = Str(dev)
					Devicedata(DetectedDevice)\PciFunction = Str(func)
					Devicedata(DetectedDevice)\VendorID = RSet(Hex(id & $FFFF),4,"0")
					Devicedata(DetectedDevice)\VendorDescr = CompanyName(id & $FFFF)
					Devicedata(DetectedDevice)\DeviceID = RSet(Hex((id >> 16) & $FFFF),4,"0")
					id = PciConfigReadLong( pci_address, $8)
					Devicedata(DetectedDevice)\Class = RSet(Hex((id >> 8) & $FFFFFF),6,"0")
					Restore ClassTable
					Repeat
						Read IDD
			        	Select IDD
			        		Case -1
			            	Break
			        		Case (id >> 8) & $FFFFFF
			        			Read DeviceData(DetectedDevice)\pClassDescr
			        			Break
			        EndSelect
			   	ForEver
					If DeviceData(DetectedDevice)\ClassDescr = ""
						DeviceData(DetectedDevice)\ClassDescr = "Not available"
					EndIf
					Devicedata(DetectedDevice)\Revision = RSet(Hex(id & $FF),2,"0")
					
					tmp = Devicedata(DetectedDevice)\VendorID + Devicedata(DetectedDevice)\DeviceID
					DeviceDescription(tmp,@ChipName,@ChipDescr)
					ChipException(tmp, tmp, Devicedata(DetectedDevice)\Revision,@North,@South)
					If North\s<>""
						ChipName\s = North\s
					EndIf
					If South\s<>""
						ChipName\s = South\s
					EndIf
					Devicedata(DetectedDevice)\DeviceDescr = ChipDescr\s + "  ( Chip : " + ChipName\s + " )"
					id = PciConfigReadLong( pci_address, $2C)
					Devicedata(DetectedDevice)\SubVendorID = RSet(Hex(id & $FFFF),4,"0")
					Devicedata(DetectedDevice)\SubVendorDescr = CompanyName(id & $FFFF)
					DeviceData(DetectedDevice)\SubDeviceID = RSet(Hex((id >> 16) & $FFFF),4,"0")
					Devicedata(DetectedDevice)\IRQ = Str(PciConfigReadChar( pci_address, $3C))
					Devicedata(DetectedDevice)\CacheLine =RSet(Hex( PciConfigReadChar(pci_address,$0C)),2,"0")
					Devicedata(DetectedDevice)\Latency = RSet(Hex(PciConfigReadChar(pci_address,$0D)),2,"0")
					Devicedata(DetectedDevice)\Header = RSet(Hex(PciConfigReadChar(pci_address,$0E)),2,"0")
					
					DetectedDevice + 1
					ReDim DeviceData.PciDevData(DetectedDevice)
				EndIf
				type = PciConfigReadChar(pci_address, $0e);
				If func =0 And (type & $80) =0 :Break: EndIf
			Next
		Next
	Next
ExitDevs:
	ProcedureReturn DetectedDevice
EndProcedure



If OpenWindow(0, 100, 200, 760, 535, "PCI devices", #PB_Window_SystemMenu | #PB_Window_MinimizeGadget | #PB_Window_ScreenCentered)
  TextGadget(0,10,5,740, 30,"Double click on a device to view details.")
  ListIconGadget(1,10,20,740,262,"Bus",50,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect)
  AddGadgetColumn(1,1,"Device",50)
  AddGadgetColumn(1,2,"Funcion",54)
  AddGadgetColumn(1,3,"Company ID",50)
  AddGadgetColumn(1,4,"Device ID",60)
  AddGadgetColumn(1,5,"Revision",70)
  AddGadgetColumn(1,6,"SubSystem",80)
  AddGadgetColumn(1,7,"Class",60)
  AddGadgetColumn(1,8,"IRQ",40)
  AddGadgetColumn(1,9,"Device Type",200)
  
  ListIconGadget(2,10,287,740,221,"Element",250,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect)
  AddGadgetColumn(2,1,"Data",460)
EndIf

LoadDeviceDriver(); Load the driver
PciElements.l = SearchPCIDevices(); Find the devices
close(); Unload the driver
For i=0 To PciElements-1
	AddGadgetItem(1,-1, DeviceData(i)\PciBus + Chr(10) + DeviceData(i)\PciDevice + Chr(10) + DeviceData(i)\PciFunction + Chr(10) + DeviceData(i)\VendorID + "h" + Chr(10) + DeviceData(i)\DeviceID + "h" + Chr(10) + DeviceData(i)\Revision + "h" + Chr(10) + DeviceData(i)\SubVendorID + DeviceData(i)\SubDeviceID + "h" + Chr(10) + DeviceData(i)\Class + "h" + Chr(10) + DeviceData(i)\IRQ + Chr(10) + DeviceData(i)\ClassDescr)
Next
Repeat
	Select WaitWindowEvent()
		Case #PB_Event_CloseWindow
			Break
	
		Case #PB_Event_Gadget
			If EventGadget() = 1
				i = GetGadgetState(1)
				If i>=0
					ClearGadgetItems(2)
					AddGadgetItem(2,-1,"Bus Number" + Chr(10) + DeviceData(i)\Pcibus)
					AddGadgetItem(2,-1,"Device Number" + Chr(10) + DeviceData(i)\PciDevice)
					AddGadgetItem(2,-1,"Function Number" + Chr(10) + DeviceData(i)\PcIFunction)
					AddGadgetItem(2,-1,"Company Code" + Chr(10) + DeviceData(i)\VendorID+"h  -> " + DeviceData(i)\VendorDescr)
					AddGadgetItem(2,-1,"Device Code" + Chr(10) + DeviceData(i)\DeviceID+"h  -> " + DeviceData(i)\DeviceDescr)
					AddGadgetItem(2,-1,"Revision Code" + Chr(10) + DeviceData(i)\Revision+"h")
					AddGadgetItem(2,-1,"Subsystem's Company Code" + Chr(10) + DeviceData(i)\SubVendorID+"h  -> " + DeviceData(i)\SubVendorDescr)
					AddGadgetItem(2,-1,"Subsystem's Device Code" + Chr(10) + DeviceData(i)\SubDeviceID+"h")
					AddGadgetItem(2,-1,"Class Code" + Chr(10) + DeviceData(i)\Class+"h")
					AddGadgetItem(2,-1,"IRQ Code" + Chr(10) + DeviceData(i)\IRQ)
					AddGadgetItem(2,-1,"Cache Line" + Chr(10) + DeviceData(i)\CacheLine+"h")
					AddGadgetItem(2,-1,"Latency" + Chr(10) + DeviceData(i)\Latency+"h")
					AddGadgetItem(2,-1,"Header" + Chr(10) + DeviceData(i)\Header+"h")
					AddGadgetItem(2,-1,"Device Type" + Chr(10) + DeviceData(i)\ClassDescr)
				EndIf
			EndIf

	EndSelect
ForEver
; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 279
; FirstLine = 249
; Folding = -
; Executable = test.exe
; CurrentDirectory = F:\120GB\WInfo.exe\
; CompileSourceDirectory